package at.fhooe.mc.magicwand.client.sensors;

import javax.microedition.io.*;

import at.fhooe.mc.magicwand.client.exceptions.CompassException;

import java.io.*;

/**
 * @class CompassSocket
 * @author Manuela Vogler
 * @brief Connection to the Symbian application which knows the compass data.
 */
public class CompassSocket {

    /**
     * The socket to the Symbian application
     */
    private SocketConnection m_con;

    /**
     * The output stream used to send messages
     */
    private DataOutputStream m_dos;

    /**
     * The input stream used to receive messages
     */
    private DataInputStream m_dis;

    /**
     * The String that stores the transmitted angle relative to the north
     */
    private String m_angle;

    /**
     * Message that is sent to the Symbian application in order to get the
     * current angle
     */
    private String m_requestString = "get";

    /**
     * Message that is sent to the Symbian application to announce that the
     * connection is going to be closed
     */
    private String m_closeString = "byebye";

    /**
     * Server's acknowledgment message
     */
    private String m_serverAck = "ok";


    /**
     * To stop receiving if the user aborts
     */
    private boolean m_receive;

    /**
     * @fn public boolean isM_receive()
     * @brief checks if receiving is active
     * @return if receiving is active
     */
    public boolean isM_receive() {
        return m_receive;
    }

    /**
     * @fn public void setM_receive(boolean _receive)
     * @brief sets receiving active or inactive
     * @param _receive set receiving active or inactive
     */
    public void setM_receive(boolean _receive) {
        m_receive = _receive;
    }

    /**
     * @fn public CompassSocket (int _port) throws CompassException
     * @brief Constructs a new CompassSocket using the given port and waits
     * for an acknowledgment from the server.
     * @param _port The port that should be used for the socket
     * @throws CompassException if connection can't be established or if server
     * doesn't send ack
     */
    public CompassSocket(int _port) throws CompassException {
        try {
            m_receive = true;
            m_con = (SocketConnection) Connector.open("socket://127.0.0.1:" + _port);
            if (!waitForAck()) { //no ack received
                throw new CompassException("Compass not available");
            }
        } catch (IOException _e) { //error while opening socket
            throw new CompassException("Compass not available");
        }
    }

    /**
     * @fn public String getAngle()
     * @brief Returns the current angle relative to the north.
     */
    public String getAngle() {
        return m_angle;
    }

    /**
     * @fn public void sendRequest() throws CompassException
     * @brief Sends a request and waits for the response.
     * @throws CompassException if transmission is not successful
     */
    public void sendRequest() throws CompassException {
        m_angle = "-1"; //reset
        sendMessage(m_requestString);
        waitForAnswer();
    }

    /**
     * @fn public void close() throws CompassException
     * @brief Sends a close message to announce the closing of the connection.
     * The connection is only closed after receiving an acknowledgment.
     * @throws CompassException if close message can't be sent, connection can't
     * be closed or if server doesn't send ack
     */
    public void close() throws CompassException {
        m_receive = true;
        sendMessage(m_closeString); //sends close message
        if (waitForAck()) { //ack received
            try {
                //close socket, output and input stream
                m_dos.close();
                m_dis.close();
                m_con.close();
            } catch (IOException ex) {
                throw new CompassException("Connection couldn't be closed");
            }
        } else { //no ack received
            throw new CompassException("Connection couldn't be closed");
        }
    }

    /**
     * @fn private void sendMessage(String _message) throws CompassException
     * @brief Sends the given message to the Symbian application.
     * @param _message Message that should be sent
     * @throws CompassException if there is a connection error
     */
    private void sendMessage(String _message) throws CompassException {
        try {
            if (m_dos == null) {
                m_dos = m_con.openDataOutputStream();
            }
            m_dos.writeUTF(_message); //sending
            m_dos.flush();
        } catch (IOException ex) {
            throw new CompassException("Error while connecting to compass");
        }
    }

    /**
     * @fn private void waitForAnswer() throws CompassException
     * @brief Waits for the server's response with the current angle and stores
     * the angle in m_angle.
     * @throws CompassException if there is a connection error
     */
    private void waitForAnswer() throws CompassException {
        String answer = receiveMessage();
        if (answer != null) {
            m_angle = answer.toString();
        } else {
            m_angle = "-1";
        }
    }

    /**
     * @fn private boolean waitForAck() throws CompassException
     * @brief Waits for the server's acknowledgment.
     * @return True if ack is received of false if not
     * @throws CompassException if there is a connection error
     */
    private boolean waitForAck() throws CompassException {
        String answer = receiveMessage();
        if (answer != null) {
            return answer.toString().equals(m_serverAck);
        } else {
            return false;
        }
    }

    /**
     * @fn private String receiveMessage() throws CompassException
     * @brief Receives a message from the server.
     * @return String with the received message
     * @throws CompassException if there is a connection error
     */
    private String receiveMessage() throws CompassException {
        StringBuffer answer = new StringBuffer();
        try {
            if (m_dis == null) {
                m_dis = m_con.openDataInputStream();
            }
            char c;
            while((c= (char)m_dis.read())!='\0' && isM_receive()) { //reads current character
                answer.append(c); //appends current character to answer as long as no \0 is received
            }
        } catch (IOException _ex) {
            throw new CompassException("Error while connecting to compass");
        }
        return answer.toString();
    }
}

