/*
 ============================================================================
 Name		: CompassReader.cpp
 Author	  : Manuela Vogler
 Version	 : 1.0
 Copyright   : 
 Description : CCompassReader implementation
 ============================================================================
 */
#include <avkon.hrh>
#include <aknnotewrappers.h>
#include <stringloader.h>
#include <f32file.h>
#include <s32file.h>
#include <aknlists.h>
#include <eikstart.h>
#include <aknmessagequerydialog.h>
#include "CompassReader.h"

/**
 * @brief Constructor for performing 1st stage construction.
 * Constructs the CCompassReader instance.
 */
CCompassReader::CCompassReader()
	{
	// No implementation required
	}

/**
 * @brief Destructor.
 * Stops listening to sensor data channel, closes all resources and destroys instance variables.
 */
CCompassReader::~CCompassReader()
	{
	//Stop listening, close channel and delete channel object.
	if (iSensrvSensorChannel)
		{
		iSensrvSensorChannel->StopDataListening();
		iSensrvSensorChannel->CloseChannel();
		}
	delete iSensrvSensorChannel;

	//Close channel info list.
	iChannelInfoList.Close();

	//Delete sensor channel finder object.
	delete iSensrvChannelFinder;
	}

/**
 * @brief Two-phased constructor.
 * Constructed instance is left on the cleanup stack.
 * @return New instance of CCompassReader
 */
CCompassReader* CCompassReader::NewLC()
	{
	CCompassReader* self = new (ELeave) CCompassReader();
	CleanupStack::PushL(self);
	self->ConstructL();
	return self;
	}

/**
 * @brief Two-phased constructor.
 * Constructed instance is not left on the cleanup stack.
 * @return New instance of CCompassReader
 */
CCompassReader* CCompassReader::NewL()
	{
	CCompassReader* self = CCompassReader::NewLC();
	CleanupStack::Pop(self);
	return self;
	}

/**
 * @brief Constructor for performing 2nd stage construction
 * Part of the construction which can cause a leave.
 */
void CCompassReader::ConstructL()
	{
	//set angle to value which indicates that no correct value has been received
	iAngle = -1;

	//construct sensor channel finder
	iSensrvChannelFinder = CSensrvChannelFinder::NewL();

	//find magnetic north data channel and start listening for new data
	GetMagneticNorthDataChannelL();

	//CompassReader is running
	iIsRunning = ETrue;
	}

/**
 * @brief Returns the current compass angle relative to the north.
 * @return The current compass angle
 */
TInt CCompassReader::GetAngle()
	{
	return iAngle;
	}

/**
 * @brief Is called when new sensor data is available. 
 * Checks if the information is about the compass data and then stores the new angle.
 * @param aChannel Sensor channel with new data
 * @param aCount Data object count in receiving buffer
 * @param aDataLost Number of lost data items
 */
void CCompassReader::DataReceived(CSensrvChannel& aChannel, TInt aCount,
		TInt aDataLost)
	{

	//if new data is from magnetic north data channel
	if (aChannel.GetChannelInfo().iChannelType
			== KSensrvChannelTypeIdMagneticNorthData)
		{
		TSensrvMagneticNorthData data;
		TPckg<TSensrvMagneticNorthData> magneticNorthPackage(data);
		aChannel.GetData(magneticNorthPackage); //get new magnetic north data

		//store angle
		iAngle = data.iAngleFromMagneticNorth;
		}
	}

/**
 * @brief Tries to find channel for magnetic north data, opens it and starts listening.
 */
void CCompassReader::GetMagneticNorthDataChannelL()
	{
	iChannelInfoList.Reset();
	TSensrvChannelInfo channelInfo;
	
	//set channel type to magnetic north data channel
	channelInfo.iChannelType = KSensrvChannelTypeIdMagneticNorthData;

	//find channels with the set properties (in this case just the magnetic north data channel)
	iSensrvChannelFinder->FindChannelsL(iChannelInfoList, channelInfo);
	if (iChannelInfoList.Count() > 0) //at least one channel found
		{
		channelInfo = iChannelInfoList[0]; //first element has to be magnetic north data channel
		iSensrvSensorChannel = CSensrvChannel::NewL(channelInfo);
		iSensrvSensorChannel->OpenChannelL();
		iSensrvSensorChannel->StartDataListeningL(this, 1, 1, 0); //start listening
		}
	iChannelInfoList.Close();
	}

/**
 * @brief Function is called if data listening failed. CompassReader state is set to not running.
 * @param aChannel Sensor channel for which data listening failed
 * @param aError The error code
 */
void CCompassReader::DataError(CSensrvChannel& aChannel,
		TSensrvErrorSeverity aError)
	{
	iIsRunning = EFalse;
	}

/**
 * @brief Checks if CompassReader is running.
 * @return ETrue if running, EFalse if not
 */
TBool CCompassReader::IsRunning()
	{
	return iIsRunning;
	}
